@@ -347,6 +347,9 @@ class DataFrame(metaclass=PureStaticDataFrameMethods):
                       provide an array copy as needed.
         """
         if np.isscalar(value):
+            if key in self._columns:
+                self._columns[key][:] = value
+                return
             value = np.full(len(self), value)
         else:
             value = np.atleast_1d(np.squeeze(np.asarray(value)))
@@ -965,7 +968,8 @@ class DataFrame(metaclass=PureStaticDataFrameMethods):
             return dfs[0].copy()
         index = dfs[0]._index
         columns = dfs[0]._columns.keys()
-        concat_columns = {k: [v] for k, v in dfs[0]._columns.items()}
+        empty = dfs[0].empty
+        concat_columns = {k: [v] if not empty else [] for k, v in dfs[0]._columns.items()}
         for df in dfs[1:]:
             if not isinstance(df, cls):
                 raise TypeError(f"Can only concatenate medvedi.DataFrame-s, got {type(df)}")
@@ -979,7 +983,8 @@ class DataFrame(metaclass=PureStaticDataFrameMethods):
         concat_df = cls()
         concat_df._index = index if not ignore_index else ()
         concat_df._columns = {
-            k: np.concatenate(v, casting="unsafe") for k, v in concat_columns.items()
+            k: np.concatenate(v, casting="unsafe") if len(v) else dfs[0]._columns[k]
+            for k, v in concat_columns.items()
         }
         return concat_df
 
@@ -1037,7 +1042,8 @@ class DataFrame(metaclass=PureStaticDataFrameMethods):
                 for i, c in enumerate(index):
                     transposed_resolved_indexes_builder[i].append(df[c])
         transposed_resolved_indexes: list[np.ndarray] = [
-            np.concatenate(vals, casting="unsafe") for vals in transposed_resolved_indexes_builder
+            np.concatenate(vals, casting="unsafe") if vals else []
+            for vals in transposed_resolved_indexes_builder
         ]
         del transposed_resolved_indexes_builder
         if len(transposed_resolved_indexes) == 1:
@@ -1068,7 +1074,10 @@ class DataFrame(metaclass=PureStaticDataFrameMethods):
             inverse_index_map[leave_mask] = np.arange(leave_mask.sum())
             inverse_map = inverse_index_map[inverse_map]
             del inverse_index_map
-        joined_columns = {i: c[index_map] for i, c in zip(indexes[0], transposed_resolved_indexes)}
+        joined_columns = {
+            i: c[index_map] if len(index_map) else c[:0]
+            for i, c in zip(indexes[0], transposed_resolved_indexes)
+        }
         pos = 0
         mask = None
         for i, (df, suffix) in enumerate(zip(dfs, suffixes)):
